module Sunscape_ext

	MAC = (Object::RUBY_PLATFORM =~ /darwin/i ? true : false)
	SA_DIR = Sketchup.find_support_file("Plugins").to_s

	SUNSCAPE_MAT = PLUGIN_NAME + " - analyze"
	SUNSCAPE_GLASS_MAT = PLUGIN_NAME + "- glass"
	MAT_DIC_NAME = "SunscapeMaterialInfo"
	ORG_MAT_KEY = "OriginalMaterial"

	class SaAppObserver < Sketchup::AppObserver
		def initialize
			add_materials Sketchup.active_model
			super
		end

	  def onNewModel(model)
	    add_materials model
	  end
	  def onOpenModel(model)
	    add_materials model
	  end

	  def add_materials model
	  	unless model.materials[SUNSCAPE_MAT]
	  		@mat_shadow_analysis = Sketchup.active_model.materials.add(SUNSCAPE_MAT)
				@mat_shadow_analysis.texture = SA_SUB_DIR + '/Resources/images/analysis.png'
			end

			unless model.materials[SUNSCAPE_GLASS_MAT]
				@mat_shadow_glass = Sketchup.active_model.materials.add(SUNSCAPE_GLASS_MAT)
				@mat_shadow_glass.texture = SA_SUB_DIR + '/Resources/images/glass.png'
			end
		end
	end

	plug_menu=UI.menu("Plugins")
	plug_menu.add_item(PLUGIN_NAME) {Sunscape_ext.ui}

	Sketchup.add_observer(SaAppObserver.new)

	@user_apikey = 'invalid'
	@dialog = nil
	@calculation_pid = nil

	UI.add_context_menu_handler do |context_menu|
		context_menu.add_separator
		sub_menu = context_menu.add_submenu(PLUGIN_NAME)

		model = Sketchup.active_model
		selection = model.selection
		materials = Sketchup.active_model.materials

		sub_menu.add_item("Use Analysis Material"){
			model.start_operation("Set Analysis Material", true)
			selection.each do |sel|
				store_org_mat sel
				sel.material = materials[SUNSCAPE_MAT]
			end
			model.commit_operation
		}

		sub_menu.add_item("Use Glass Material"){
			model.start_operation("Set Glass Material", true)
			selection.each do |sel|
				store_org_mat sel
				sel.material = materials[SUNSCAPE_GLASS_MAT]
			end
			model.commit_operation
		}

		sub_menu.add_item("Reset Material"){
			model.start_operation("Set Glass Material", true)
			selection.each do |sel|
				dic = sel.attribute_dictionary(MAT_DIC_NAME, false)
				if dic && materials[dic[ORG_MAT_KEY]]
					sel.material = materials[dic[ORG_MAT_KEY]]
				else
					sel.material = nil
				end
			end
			model.commit_operation
		}
	end

	def Sunscape_ext.store_org_mat entity
		materials = Sketchup.active_model.materials
		if entity.material && entity.material != materials[SUNSCAPE_MAT] && entity.material != materials[SUNSCAPE_GLASS_MAT]
			dic = entity.attribute_dictionary(MAT_DIC_NAME, true)
			dic[ORG_MAT_KEY] = entity.material.name
		end
	end

	def Sunscape_ext.ui

		if @dialog == nil || !@dialog.visible?
			#UI
			@dialog = UI::HtmlDialog.new({
			  :dialog_title => PLUGIN_NAME,
			  :preferences_key => "shadow.analysis.2",
			  :scrollable => true,
			  :resizable => true,
			  :width => 480,
			  :height => 700,
			  :min_width => 400,
			  :min_height => 500,
			  # :max_width => 480,
			  # :max_height => 1000,
			  :style => UI::HtmlDialog::STYLE_UTILITY
			})

			ui_file = SA_SUB_DIR + '/Resources/ui/html/ui.html'
			ui_html = File.read(ui_file)

			translated_ui = Sunscape_ext.translate ui_html
			# puts translated_ui

			@dialog.set_html translated_ui

			@dialog.add_action_callback('start') {|action_context, settings|
				valid, msg = Sunscape_ext.valid_license? @user_apikey
				if valid
					Sunscape_ext.main(settings)
				else
					command = ''
					command += "$('#SettingsModal').modal('show');"
					@dialog.execute_script(command)
				end
				#@dialog.close
			}

			@dialog.add_action_callback('close') {|action_context|
				@dialog.close
				@dialog = nil
	    		return 0
			}

			@dialog.add_action_callback('stop_calculation') {|action_context|
				Process.kill("KILL", @calculation_pid) if @calculation_pid #tested on windows
			}

			@dialog.add_action_callback('load_settings') {|action_context|
				user_settings = JSON.parse(Sketchup.read_default('Sunscape', 'Settings', '{}'))
				command = ''
				if user_settings.key?('debug')
					command += '$("#debug").prop("checked",' + user_settings['debug'].to_s + ');'
					@debug_mode = user_settings['debug']
				end
				if user_settings.key?('ApiKey')
					command += '$("#ApiKey").val("' + user_settings['ApiKey'] +'");'
					@user_apikey = user_settings['ApiKey']
				else
					command += "$('#SettingsModal').modal('show');"
				end
				command += "validate_settings();"
				# puts command
				@dialog.execute_script(command)
			}

			@dialog.add_action_callback('save') {|action_context, settings|
				user_settings = JSON.parse(settings)
				if user_settings.key?('ApiKey')
					@user_apikey = user_settings['ApiKey']
				end
				if user_settings.key?('debug')
					@debug_mode = user_settings['debug']
				end
				text = settings.dump
				Sketchup.write_default('Sunscape', 'Settings', text[1, text.size - 2])
				command = "$('#SettingsModal').modal('hide');"
				@dialog.execute_script(command)
			}

			@dialog.add_action_callback('validate') {|action_context, apikey|
				valid, msg = Sunscape_ext.valid_license? apikey
				@user_apikey = apikey if valid
				command = "validation_result('ApiKey'," + valid.to_s + ", \"" + msg + "\");"
				# puts command
				@dialog.execute_script(command)
			}

			@dialog.add_action_callback('call_to_action'){|action_context, url|
				UI.openURL url
			}

			@dialog.add_action_callback('open_file'){|action_context|
				analysis_flie = UI.openpanel("Open saved analysis", "", "Sunscape *.sa|*.sa||")
				cmd = '"' + SA_SUB_DIR + '/Sunscape/Sunscape.exe" "' + analysis_flie + '"'
				pid = spawn(cmd)
				Process.detach(pid)
			}

			@dialog.add_action_callback('send_latitude'){|action_context|
				model_lat = Sketchup.active_model.shadow_info['Latitude']
				puts model_lat


				@dialog.execute_script('$("#Latitude").val("' + model_lat.to_s + '");');
			}

			@dialog.show
		else
			@dialog.bring_to_front
		end
	end

	def Sunscape_ext.main settings
		model = Sketchup.active_model
		camera = model.active_view.camera

		options = {
			camera: {
				eye: [camera.eye.x.to_f, camera.eye.y.to_f, camera.eye.z.to_f],
				target: [camera.target.x.to_f, camera.target.y.to_f, camera.target.z.to_f],
				up: [camera.up.x.to_f, camera.up.y.to_f, camera.up.z.to_f],
			},
			location: {
				lat: model.shadow_info['Latitude'],
				lon: model.shadow_info['Longitude'],
				north: model.shadow_info["NorthAngle"]
			},

			apikey: @user_apikey

		}

		geometry = 's { "Calculation": ' + settings + ', "Options": ' + options.to_json + "}\n"
		# puts geometry
		# getting selected models
		selection = Sketchup.active_model.selection
		faces = []
		# TODO: reading Components geometry
		vert_no = 1

		# model.entities.each do |entity|
		# 	if entity.class == Sketchup::Face
		# 		faces << entity
		# 	elsif entity.class == Sketchup::Group
		# 		faces += getFaces(entity.entities)

		model.start_operation('Reading geometry', false, true, true)
		Sketchup::set_status_text("Reading geometry...")

		faces += getFaces model.entities, nil

		#faces.each do |face_info|
		output = File.new(SA_DIR + '/geometry.osa', 'w+')

		s = Time.now
		face_count = 0
		for face_info in faces

			face_count += 1
			Sketchup::set_status_text("Reading geometry:    " + face_count.to_s + "/" + faces.size.to_s)

			face = face_info['f']
			transformation = face_info['t']
			material = face_info['m']

			face.material = material if material != nil && !face.material

			# managing if entity is selected or no
		  face_option = selected? face
		  geometry += face_option ? "o " + face_option + "\n" : "o turn off\n"

			mesh = face.mesh

			if transformation != nil
				mesh.transform! transformation
			end

			verts = []

			#for polygon in mesh.polygons
			for i in 1..mesh.count_polygons
				geometry += "\n"
				f_string = ''
				for pt in mesh.polygon_points_at(i)
					geometry += 'v ' + pt.x.to_f.to_s + ' ' + pt.y.to_f.to_s + ' ' + pt.z.to_f.to_s + "\n"
					f_string += vert_no.to_s + '// '

					vert_no += 1
				end
				f = 'f '
				f += f_string
				geometry += f.strip! + "\n"
			end

			output.write geometry
			geometry = ''
		end

		model.commit_operation
		Sketchup::set_status_text("Reading geometry completed in " + (Time.now - s).to_s + "s.")
		output.close
		engine_path = SA_SUB_DIR + '/Sunscape/'
		executable = @debug_mode ? 'Sunscape-console.exe' : 'Sunscape.exe'

		if(File.exist?(engine_path + executable))
			cmd = '"' + engine_path + executable + '" "' + SA_DIR + '/geometry.osa"'
			pid = spawn(cmd)
			@calculation_pid = pid
			Process.detach(pid)
		else
			UI.messagebox("Error! Couldn't find Sunscape calculation engine.", MB_OK)
		end

	end

	def Sunscape_ext.getFaces entities, transformation = nil, material = nil
		faces = []
		#entities.each do |entity|
		for entity in entities
			mat = material
			if entity.class == Sketchup::Face && entity.deleted? === false && entity.visible? === true
				# add plane/normal check here
				mat = entity.material if entity.material
				faces << {'f' => entity, 't' => transformation, 'm' => mat}
			elsif entity.class == Sketchup::Group && entity.deleted? === false && entity.deleted? === false && entity.visible? === true
				if transformation != nil
					trans = transformation * entity.transformation
				else
					trans = entity.transformation
				end
				faces += getFaces entity.entities, trans, entity.material
			elsif entity.class == Sketchup::ComponentInstance && entity.deleted? === false && entity.visible? === true

				if transformation != nil
					trans = transformation * entity.transformation
				else
					trans = entity.transformation
				end
				faces += getFaces entity.definition.entities, trans, entity.material
			end
		end
		return faces
	end

	def Sunscape_ext.selected? face
		selection = Sketchup.active_model.selection
		# puts "material selection" if face.material && (face.material.name == SUNSCAPE_MAT || face.material.name == SUNSCAPE_GLASS_MAT)
		return "selected" if selection.include?(face)
		return "selected" if face.material && face.material.name == SUNSCAPE_MAT
		return "glazing" if face.material && face.material.name == SUNSCAPE_GLASS_MAT
		false
	end

	def Sunscape_ext.valid_license? apikey
		response = backend_call('api/shadow_subscription', {:apikey => apikey, :session => "subscription_check"})

		body = JSON.parse(response.body)


		case response.code
			when '200'
				return true, 'License is valid'
			when '400'
				msg = "Unknown API key. If the problem persists please contact us at office@deltacodes.pl."
				#result = UI.messagebox(msg, MB_OK)
				return false, msg
			when '401'
				msg = "Invalid session ID."
				#result = UI.messagebox(msg, MB_OK)
				return false, msg
			when '402'
				msg = "You don't have a valid Shadow Analysis subscription. Please visit <a href='skp:call_to_action@https://deltacodes.eu/'>https://deltacodes.eu</a> to accuire one."
				#result = UI.messagebox(msg, MB_OK)
				return false, msg
		end
		return false, 'Error while checking the license. Please contact us at office@deltacodes.pl'
	end

	def Sunscape_ext.translate text
		Sketchup::require SA_SUB_DIR + '/Resources/en-US/strings'

		translated = ''
		chars = text.chars
		pattern = false
		word = ''
		chars.each do |char|
			# kjidewjiewq .t[slowa] dwqfqrw
			if pattern == true
				if (word == '.' && char != 't') ||
					(word == '.t' && char != '[')
					pattern = false
					translated += word + char
					word = ''
				elsif char == ']'
					# translate
					word = word[3, word.length - 3]

					if word == 'base_path'
						translated_word = SA_SUB_DIR + '/Resources/ui/'
					else
						translated_word = localize word
					end
					#puts translated_word

					translated += translated_word
					word = ''
					pattern = false
				else
					word += char
				end
			elsif char == '.'
				pattern = true
				word += char
			else
				translated += char
			end
		end

		return translated
	end

	require 'uri'
	require 'net/http'

	def Sunscape_ext.backend_call(action, device_hash = {})
    backend_service = 'https://pro.solary.org/';
    url = backend_service + action;

    begin
      if device_hash.empty?
        curl_response = open(url)
      else
        #ins = device_hash.to_s;
        uri = URI(url)

        http = Net::HTTP.new(uri.host, uri.port)
        http.use_ssl = true
        request = Net::HTTP::Post.new(uri.path, {'Content-Type' =>'application/json'})
        request.body = device_hash.to_json
        curl_response = http.request(request)



      end
    rescue
      msg = "Unable to check your license. Please check your internet connection."
      UI.messagebox(msg, MB_OK)
      # TODO: error message box
    end

    return curl_response;
  end

end


